package app

import (
	"database/sql"

	"code.tnxs.net/taxnexus/lib/api/geo/geo_models"
)

// TaxTypeActivityWrapper wraps the object with workflow params
type TaxTypeActivityWrapper struct {
	Obj      geo_models.TaxType
	APIKey   string
	SagaID   string
	SagaType string
}

// TaxType is a first class object type
type TaxType struct {
	ID                 string
	Account            Account
	AccountID          string
	AccountingRuleCode string
	Active             bool
	AgencyType         string
	AgentID            string
	Amount             float64
	Category           string
	CollectorDomainID  string
	CompanyID          string
	Contact            Contact
	ContactID          string
	CreatedByID        string
	CreatedDate        sql.NullTime
	Description        string
	EffectiveDate      sql.NullTime
	EndDate            sql.NullTime
	EnrollmentStatus   string
	FilingCity         string
	FilingCountry      string
	FilingEmail        string
	FilingMethod       string
	FilingPostalcode   string
	FilingState        string
	FilingStreet       string
	Formatted          TaxTypeFormatted
	Fractional         bool
	Frequency          string
	GeocodeString      string
	InterestRate       float64
	IsMedicinal        bool
	IsRecreational     bool
	LastModifiedByID   string
	LastModifiedDate   sql.NullTime
	MarkupRate         float64
	Name               string
	OwnerID            string
	Passthrough        bool
	PenaltyDays        int64
	PenaltyRate        float64
	Rate               float64
	Reference          string
	RevenueBase        float64
	RevenueNet         float64
	RevenueNotTaxable  float64
	SalesRegulation    string
	Status             string
	TaxnexusCodeID     string
	TaxnexusNumber     string
	TemplateID         string
	UnitBase           float64
	Units              string
}

// TaxTypeFormatted offers formatted versions of numerical values
type TaxTypeFormatted struct {
	CreatedDate      string
	EffectiveDate    string
	EndDate          string
	Fractional       string
	InterestRate     string
	IsMedicinal      string
	IsRecreational   string
	LastModifiedDate string
	MarkupRate       string
	PassThrough      string
	PenaltyDays      string
	PenaltyRate      string
	Rate             string
}
